
/* GnomeMeeting -- A Video-Conferencing application
 * Copyright (C) 2000-2004 Damien Sandras
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * GnomeMeting is licensed under the GPL license and as a special exception,
 * you have permission to link or otherwise combine this program with the
 * programs OpenH323 and Pwlib, and distribute the combination, without
 * applying the requirements of the GNU GPL to the OpenH323 program, as long
 * as you do follow the requirements of the GNU GPL for all the rest of the
 * software thus combined.
 */

/*
 *                         dbus_helper.cpp  -  description
 *                         --------------------------
 *   begin                : Wed 19 Oct 2005
 *   copyright            : (C) 2005 by Julien Puydt
 *   description          : Implementation of a DBUS helper.
 *
 */

#include <stdlib.h>
#define DBUS_API_SUBJECT_TO_CHANGE
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-lowlevel.h>

/* Here is the description of method calls that this little program
 * manages for gnomemeeting:
 *
 * "GetVersion"
 * in    : nil
 * out   : int (version of the dbus api)
 * 
 * "GetSupportedProtocols"
 * in    : nil
 * out   : list of strings (supported protocols, example: "SIP" and "H.323")
 *
 */

#define GM_INFO_SERVICE   "net.gnomemeeting.helper"
#define GM_INFO_INTERFACE "net.gnomemeeting.helper"
#define GM_HELPER_OBJECT  "/net/gnomemeeting/helper"

/* this function is called by DBUS when a watched type of message
 * arrives ; it is only used to know if we're still connected to the bus
 */
static DBusHandlerResult filter_func (DBusConnection *connection,
				      DBusMessage *message,
				      void *user_data);


/* this function dispatches the message to the other helpers */
static DBusHandlerResult path_message_func (DBusConnection *connection,
					    DBusMessage *message,
					    void *user_data);


/* those are the helper functions that take care of the actual handling */
static void handle_get_version (DBusConnection *connection,
				DBusMessage *message);
static void handle_get_supported_protocols (DBusConnection *connection,
					    DBusMessage *message);

static DBusObjectPathVTable call_vtable = {
  NULL,
  path_message_func,
  NULL,
};

/* real implementation */

static DBusHandlerResult
filter_func (DBusConnection *connection,
	     DBusMessage *message,
	     void *user_data)
{
  if (dbus_message_is_signal (message,
			      DBUS_INTERFACE_LOCAL,
			      "Disconnected")) {

    exit (-1);
  }

  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}


static DBusHandlerResult
path_message_func (DBusConnection *connection,
		   DBusMessage *message,
		   void *user_data)
{
  DBusHandlerResult result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;

  if (dbus_message_is_method_call (message,
				   GM_INFO_SERVICE,
				   "GetVersion")) {

    handle_get_version (connection, message);
    result = DBUS_HANDLER_RESULT_HANDLED;
  }
  else if (dbus_message_is_method_call (message,
					GM_INFO_SERVICE,
					"GetSupportedProtocols")) {

    handle_get_supported_protocols (connection, message);
    result = DBUS_HANDLER_RESULT_HANDLED;
  }

  return result;
}

static void
handle_get_version (DBusConnection *connection,
		    DBusMessage *message)
{
  DBusMessage *reply = NULL;
  const gchar *version = VERSION;

  reply = dbus_message_new_method_return (message);
  (void)dbus_message_append_args (reply,
				  DBUS_TYPE_STRING, &version,
				  DBUS_TYPE_INVALID);
  (void)dbus_connection_send (connection, reply, NULL);
  dbus_connection_flush (connection);
  dbus_message_unref (reply);
}

static void
handle_get_supported_protocols (DBusConnection *connection,
				DBusMessage *message)
{
  DBusMessage *reply = NULL;
  const gchar *array [] = { "H.323", "SIP" };
  const gchar **v_array = array;

  reply = dbus_message_new_method_return (message);
  (void)dbus_message_append_args (reply,
				  DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &v_array, 2,
				  DBUS_TYPE_INVALID);
  (void)dbus_connection_send (connection, reply, NULL);
  dbus_connection_flush (connection);
  dbus_message_unref (reply);
}

/* well, it's needed anyway */
int
main (int argc, char *argv[])
{
  DBusConnection *connection = NULL;
  GMainLoop *loop = NULL;

  connection = dbus_bus_get (DBUS_BUS_SESSION, NULL);
  if (connection == NULL) {

    exit (-1);
  }

  if (!dbus_connection_add_filter (connection, filter_func, NULL, NULL)) {

    dbus_connection_disconnect (connection);
    exit (-1);
  }

  if (!dbus_connection_register_object_path (connection,
					     GM_HELPER_OBJECT,
					     &call_vtable, NULL)) {

    dbus_connection_disconnect (connection);
    exit (-1);
  }

  if (dbus_bus_request_name (connection,
			     GM_INFO_SERVICE, 0, NULL) == -1) {

    dbus_connection_disconnect (connection);
    exit (-1);
  }

  loop = g_main_loop_new (NULL, FALSE);
  dbus_connection_setup_with_g_main (connection,
				     g_main_loop_get_context (loop));

  g_main_loop_run (loop);
}
